# Pyntor::Slides - Display slides
# Copyright (C) 2004 Josef Spillner <josef@coolprojects.org>
# Published under GNU GPL conditions

import pygame
from pygame.locals import *
import sys
import time
import random
import os, pwd
import re

class Slides:
	def run(self, options):
		""" Options """

		slideshow = options[0]

		if len(options) > 1:
			option = options[1]
			if option.find("-") >= 0:
				(start, stop) = option.split("-")
				readslide = int(start)
				lastslide = int(stop)
			else:
				readslide = int(option)
				lastslide = -1
		else:
				readslide = 1
				lastslide = -1

		""" Screen setup """

		screen = pygame.display.get_surface()

		screenorig = pygame.Surface((screen.get_width(), screen.get_height()))
		screenorig.blit(screen, (0, 0))

		white = pygame.Surface((screen.get_width(), screen.get_height()))
		white.fill((255, 255, 255))
		#alphapart = 160
		alphapart = 0

		screencopy = pygame.Surface((screen.get_width(), screen.get_height()))
		screencopy.blit(screen, (0, 0))
		screencopy2 = pygame.Surface((screen.get_width(), screen.get_height()))
		screencopy2.blit(screen, (0, 0))

		surface = pygame.Surface((screen.get_width(), screen.get_height()))

		pygame.event.clear()

		titlefont = pygame.font.SysFont("Vera Sans", 60)
		bigfont = pygame.font.SysFont("Vera Sans", 42)
		smallfont = pygame.font.SysFont("Vera Sans", 24)

		""" Read in slide file """

		f = open(slideshow)
		blines = f.readlines()
		f.close()
		lines = []
		for line in blines:
			lines.append(unicode(line.rstrip(), "utf-8"))

		oldslide = 0
		marker = -1
		mouse = 0
		draw = 0
		dodraw = 0
		posx = 0
		posy = 0
		dirty = 0

		eventlist = {}

		while 1:
			""" Control """

			pygame.event.pump()
			event = pygame.event.poll()
			if event.type == KEYDOWN:
				key = event.key
				if key == K_ESCAPE or pygame.event.peek(QUIT):
					break
				if key == K_RETURN:
					readslide = oldslide + 1
					marker = -1
				if key == K_BACKSPACE:
					readslide = oldslide - 1
					marker = -1
				if key == K_f:
					mouse = 1 - mouse
					pygame.mouse.set_visible(mouse)
					pygame.display.toggle_fullscreen()
				if key == K_m:
					mouse = 1 - mouse
					pygame.mouse.set_visible(mouse)
				if key == K_s:
					os.mkdir("static")
					filename = "slide" + str(oldslide) + ".bmp"
					pygame.image.save(screen, os.path.join("static", filename))
				if key == K_PLUS:
					alphapart += 15
					white.set_alpha(alphapart)
					screencopy.blit(screenorig, (0, 0))
					screencopy.blit(white, (0, 0))
					screencopy2.blit(screenorig, (0, 0))
					screencopy2.blit(white, (0, 0))
					readslide = oldslide
				if key == K_MINUS:
					alphapart -= 15
					white.set_alpha(alphapart)
					screencopy.blit(screenorig, (0, 0))
					screencopy.blit(white, (0, 0))
					screencopy2.blit(screenorig, (0, 0))
					screencopy2.blit(white, (0, 0))
					readslide = oldslide
				if key == K_DOWN:
					marker += 1
					readslide = oldslide
				if key == K_UP:
					marker -= 1
					readslide = oldslide

			if mouse:
				if event.type == MOUSEBUTTONDOWN:
					draw = 1
					dodraw = 0
					(posx, posy) = event.pos
					line = (posy - 100) / 30
					if eventlist.has_key(line):
						if eventlist[line].find("http") >= 0:
							os.spawnlp(os.P_NOWAIT, "konqueror", "konqueror", eventlist[line])
						else:
							os.spawnlp(os.P_NOWAIT, eventlist[line], eventlist[line])
				if event.type == MOUSEBUTTONUP:
					draw = 0
					dirty = 1
				if event.type == MOUSEMOTION:
					(posx, posy) = event.pos
					dodraw = 1

			if draw:
				if dodraw:
					screencopy2.fill((255, 100, 0), ((posx, posy), (20, 20)))
			if dirty:
				dirty = 0
				readslide = oldslide

			if readslide:
				heading = ""
				subheading = ""
				pageheading = ""
				#counter = 1
				counter = 0
				rawmode = 0
				rawlist = []
				eventlist = {}

				oldline = ""
				for l in lines:
					if l.find("======") != -1:
						heading = oldline
						counter = 1
					if oldline.find("======") != -1:
						subheading = l
					if readslide > 1 or readslide == 1:
						if l.find("-----") != -1:
							counter += 1
							if counter == readslide:
								pageheading = oldline
								rawmode = 1
							else:
								rawmode = 0
					oldline = l

					if rawmode == 1:
						rawlist.append(l)

				if readslide != oldslide:
					surface.blit(screencopy, (0, 0))
					screencopy2.blit(screencopy, (0, 0))
				else:
					surface.blit(screencopy2, (0, 0))

				if readslide == 1 and heading is not "":
					f = titlefont.render(heading, 1, (255, 0, 0))
					surface.blit(f, ((surface.get_width() - f.get_width()) / 2, 200))
					f = smallfont.render(subheading, 1, (0, 0, 0))
					surface.blit(f, ((surface.get_width() - f.get_width()) / 2, 300))
				else:
					if len(rawlist) == 0:
						break
					if readslide == lastslide + 1:
						break

					f = bigfont.render(pageheading, 1, (0, 0, 250))
					surface.blit(f, (50, 50))

					i = 0
					rawlist.pop()
					rawlist.pop(0)
					for l in rawlist:
						reimg = re.compile("\[img:(\S+)\]")
						mimg = reimg.match(l)
						reurl = re.compile("\[url:(\S+)\]")
						murl = reurl.match(l)
						reexec = re.compile("\[exec:(\S+)\]")
						mexec = reexec.match(l)
						if mimg:
							img = mimg.group(1)
							pic = pygame.image.load(img)
							surface.blit(pic, (50, 100 + i * 30))
							i += pic.get_height() / 30 + 1
						elif murl:
							url = murl.group(1)
							smallfont.set_italic(1)
							f = smallfont.render(url, 1, (0, 0, 150))
							smallfont.set_italic(0)
							surface.blit(f, (50, 100 + i * 30))
							eventlist[i] = url
							i += 1
						elif mexec:
							execurl = mexec.group(1)
							lmod = re.sub(reexec, execurl, l)
							smallfont.set_italic(1)
							f = smallfont.render(lmod, 1, (0, 0, 0))
							surface.blit(f, (50, 100 + i * 30))
							f = smallfont.render(execurl, 1, (0, 150, 150))
							smallfont.set_italic(0)
							surface.blit(f, (50, 100 + i * 30))
							eventlist[i] = execurl
							i += 1
						else:
							if marker == -1 or marker == i:
								f = smallfont.render(l, 1, (0, 0, 0))
							else:
								f = smallfont.render(l, 1, (100, 100, 100))
							surface.blit(f, (50, 100 + i * 30))
							i += 1

					f = smallfont.render(subheading + " (" + str(readslide) + ")", 1, (0, 0, 0))
					surface.blit(f, (50, 700))

				oldslide = readslide
				readslide = 0

				screen.blit(surface, (0, 0))
				pygame.display.flip()

		screen.blit(screencopy, (0, 0))

component = Slides()

